<?php

/**
 * @file
 * API documentation for Khalti module.
 *
 * This file provides documentation for hooks exposed by the Khalti module.
 * Modules can implement these hooks to alter or extend the functionality
 * provided by the Khalti module.
 */

/**
 * Alter the customer information array for Khalti transactions.
 *
 * This hook allows other modules to modify the customer information
 * used in Khalti payment configurations.
 *
 * Example:
 * ```
 * function my_module_khalti_customer_info_alter(array &$customer_info) {
 *   $customer_info['name'] = 'John doe'; // Dynamically set the name.
 *   $customer_info['phone'] = '9800000001'; // Dynamically set the phone number.
 *   $customer_info['email'] = 'updated@example.com'; // Optionally modify email.
 * }
 * ```
 *
 * @param array $customer_info
 *   An associative array of customer information passed to Khalti, containing:
 *   - name: (string) The customer's name.
 *   - email: (string) The customer's email address.
 *   - phone: (string) The customer's phone number.
 *
 * @see https://docs.khalti.com for API integration details.
 */
function hook_khalti_customer_info_alter(array &$customer_info) {
  // Modify the customer information array as needed.
}
