<?php

namespace Drupal\khalti\PluginForm;

use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\khalti\Khalti;

/**
 * Defines the CommerceStripeCheckoutCheckoutForm class.
 *
 * Extends the BasePaymentOffsiteForm class for handling checkout forms.
 */
class KhaltiCheckoutCheckoutForm extends BasePaymentOffsiteForm {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;

    /** @var \Drupal\khalti\Plugin\Commerce\PaymentGateway $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $configuration = $payment_gateway_plugin->getConfiguration();
    $mode = $configuration['mode'];
    $pass_customer_email = $configuration['pass_customer_email'] ?? FALSE;

    $order = $payment->getOrder();
    $token = khalti_random_token(16);

    $khalti_sess = [
      'khalti_payment' => $payment,
      'khalti_order' => $order,
      'mode' => $mode,
      'token' => $token,
    ];

    khalti_set_session_data('khalti_sess', $khalti_sess);

    $khalti_data = $payment_gateway_plugin->setKhaltiCheckoutCheckoutData($payment);

    $order = $khalti_data['order'];
    $currency = $khalti_data['currency'];

    if ($currency == 'NPR') {
      $line_items_data = [];

      try {
        $total_amount = 0;

        if (!empty($order)) {
          global $base_url;

          // Get the total price from the order.
          $total_price = $order->getTotalPrice();

          $order_id = $order->id();

          $total_amount = $total_price->getNumber();

          $success_redirect_url = Url::fromRoute('khalti.payment_success', ['token' => $token], ['absolute' => TRUE])->toString();
          $cancel_redirect_url = Url::fromRoute('khalti.payment_cancel', [], ['absolute' => TRUE])->toString();

          if($mode == 'live') {
            $secret_key = $configuration['secret_key'] ?? '';
          } else {
            $secret_key = $configuration['test_secret_key'] ?? '';
          }

          $khalti = \Drupal::service('khalti.khalti');

          $billing_profile = $order->getBillingProfile();
          $customer_name = '';
          // Check if the billing profile exists.
          if ($billing_profile) {
            // Get address fields from the billing profile.
            $address_field = $billing_profile->get('address')->first();
            $customer_name = $address_field->given_name .' '.$address_field->family_name;
          }

          $order_email = $order->get('mail')->value;
          $order_name = 'Order #'.$order->id();

          $khalti->setSecretKey($secret_key);

          $customer_info = [
            'name' => $customer_name,
            'email' => $order_email,
            'phone' => '', // Default phone value.
          ];

          // Allow other modules to alter the customer information.
           \Drupal::moduleHandler()->invokeAll('khalti_customer_info_alter', [&$customer_info]);

          $khalti->config(
          // Return URL.
             $success_redirect_url,
          // Website URL.
             $base_url,
          // Amount in ruppes.
             $total_amount,
          // Purchase Order ID.
             $order_id,
          // Purchase Order Name.
             $order_name,
          // Customer Information.
             $customer_info
          );

          if ($mode == 'live') {
            // Enable for production.
            $khalti->init(TRUE);
          } else {
            // Process the payment.
            $khalti->init();
          }
        }
      } catch (\Exception $e) {
        $message = $e->getMessage();
        \Drupal::messenger()->addError(t($message));
      }
    } else {
      $message = 'Please contact the site administrator. Khalti supports payments in Nepalese Rupees (NPR) only.';
      \Drupal::messenger()->addError(t($message));
    }
  }

}
